/**
 * Copyright (C) oStorybook Team
 *
 * This program is free software, see <http://www.gnu.org/licenses/>.
 *
 * Parts of this code are from the SHEF project developed and published by Bob Tantlinger.
 */
package shef.actions;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.Action;
import javax.swing.KeyStroke;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.Document;
import javax.swing.text.TextAction;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoManager;
import javax.swing.undo.UndoableEdit;
import shef.tools.LOG;
import i18n.I18N;
import resources.icons.ICONS;
import resources.icons.IconUtil;

/**
 * Manages compound undoable edits.
 *
 * Before an undoable edit happens on a particular document, you should call the static method
 * CompoundUndoManager.beginCompoundEdit(doc)
 *
 * Conversely after an undoable edit happens on a particular document, you shoulc call the static method
 * CompoundUndoManager.beginCompoundEdit(doc)
 *
 * For either of these methods to work, you must add an instance of CompoundUndoManager as a document listener... e.g
 *
 * doc.addUndoableEditListener(new CompoundUndoManager(doc, new UndoManager());
 *
 * Note that each CompoundUndoManager should have its own UndoManager.
 *
 *
 * @author Bob Tantlinger
 */
public class CompoundUndoManager implements UndoableEditListener {

	/**
	 * Static undo action that works across all documents with a CompoundUndoManager registered as an
	 * UndoableEditListener
	 */
	public static Action ActionUndo = new UndoAction();

	/**
	 * Static undo action that works across all documents with a CompoundUndoManager registered as an
	 * UndoableEditListener
	 */
	public static Action ActionRedo = new RedoAction();

	private UndoManager undoer;
	private CompoundEdit compoundEdit = null;
	private Document document = null;
	private static List<Document> docs = new ArrayList<>();
	private static List<CompoundUndoManager> lsts = new ArrayList<>();
	private static List<UndoManager> undoers = new ArrayList<>();

	protected static void registerDocument(Document doc, CompoundUndoManager lst, UndoManager um) {
		docs.add(doc);
		lsts.add(lst);
		undoers.add(um);
	}

	/**
	 * Gets the undo manager for a document that has a CompoundUndoManager as an UndoableEditListener
	 *
	 * @param doc
	 * @return The registed undomanger for the document
	 */
	public static UndoManager getUndoManagerForDocument(Document doc) {
		if (undoers != null) {
			for (int i = 0; i < docs.size(); i++) {
				if (docs.get(i) == doc) {
					return (UndoManager) undoers.get(i);
				}
			}
		}

		return null;
	}

	/**
	 * Notifies the CompoundUndoManager for the specified Document that a compound edit is about to begin.
	 *
	 * @param doc
	 */
	public static void beginCompoundEdit(Document doc) {
		for (int i = 0; i < docs.size(); i++) {
			if (docs.get(i) == doc) {
				CompoundUndoManager l = (CompoundUndoManager) lsts.get(i);
				l.beginCompoundEdit();
				return;
			}
		}
	}

	/**
	 * Notifies the CompoundUndoManager for the specified Document that a compound edit is complete.
	 *
	 * @param doc
	 */
	public static void endCompoundEdit(Document doc) {
		for (int i = 0; i < docs.size(); i++) {
			if (docs.get(i) == doc) {
				CompoundUndoManager l = (CompoundUndoManager) lsts.get(i);
				l.endCompoundEdit();
				return;
			}
		}
	}

	/**
	 * Updates the enabled states of the ActionUndo and ActionRedo actions for the specified document
	 *
	 * @param doc
	 */
	public static void updateUndo(Document doc) {
		UndoManager um = getUndoManagerForDocument(doc);
		if (um != null) {
			ActionUndo.setEnabled(um.canUndo());
			ActionRedo.setEnabled(um.canRedo());
		}
	}

	/**
	 * Discards all edits for the specified Document
	 *
	 * @param doc
	 */
	public static void discardAllEdits(Document doc) {
		UndoManager um = getUndoManagerForDocument(doc);
		if (um != null) {
			um.discardAllEdits();
			ActionUndo.setEnabled(um.canUndo());
			ActionRedo.setEnabled(um.canRedo());
		}
	}

	/**
	 * Creates a new CompoundUndoManager
	 *
	 * @param doc
	 * @param um The UndoManager to use for this document
	 */
	public CompoundUndoManager(Document doc, UndoManager um) {
		undoer = um;
		document = doc;
		registerDocument(document, this, undoer);
	}

	/**
	 * Creates a new CompoundUndoManager
	 *
	 * @param doc
	 */
	public CompoundUndoManager(Document doc) {
		this(doc, new UndoManager());
	}

	@Override
	public void undoableEditHappened(UndoableEditEvent evt) {
		UndoableEdit edit = evt.getEdit();
		if (compoundEdit != null) {
			compoundEdit.addEdit(edit);
		} else {
			undoer.addEdit(edit);
			updateUndo(document);
		}
	}

	protected void beginCompoundEdit() {
		compoundEdit = new CompoundEdit();
	}

	protected void endCompoundEdit() {
		if (compoundEdit != null) {
			compoundEdit.end();
			undoer.addEdit(compoundEdit);
			updateUndo(document);
		}
		compoundEdit = null;
	}

	static class UndoAction extends TextAction {

		public UndoAction() {
			super(I18N.getMsg("shef.undo"));
			putValue(Action.SMALL_ICON, IconUtil.getIconSmall(ICONS.K.EDIT_UNDO));
			putValue(MNEMONIC_KEY, I18N.getMnem("shef.undo"));
			setEnabled(false);
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_DOWN_MASK));
			putValue(SHORT_DESCRIPTION, getValue(NAME));
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			Document doc = getTextComponent(e).getDocument();
			UndoManager um = getUndoManagerForDocument(doc);
			if (um != null) {
				try {
					um.undo();
				} catch (CannotUndoException ex) {
					LOG.error("Unable to undo:", ex);
				}
				updateUndo(doc);
			}
		}
	}

	static class RedoAction extends TextAction {

		public RedoAction() {
			super(I18N.getMsg("shef.redo"));
			putValue(Action.SMALL_ICON, IconUtil.getIconSmall(ICONS.K.EDIT_REDO));
			putValue(MNEMONIC_KEY, I18N.getMnem("shef.redo"));
			setEnabled(false);
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_DOWN_MASK));
			putValue(SHORT_DESCRIPTION, getValue(NAME));
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			Document doc = getTextComponent(e).getDocument();
			UndoManager um = getUndoManagerForDocument(doc);
			if (um != null) {
				try {
					um.redo();
				} catch (CannotUndoException ex) {
					LOG.error("Unable to redo", ex);
				}
				updateUndo(doc);
			}
		}
	}
}
