/*
 * Copyright (C) 2020 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a fileCopy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.toolkit.xml;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import i18n.I18N;
import storybook.model.hbn.entity.Attribute;
import storybook.toolkit.LOG;
import storybook.toolkit.file.IOUtil;
import storybook.ui.dialog.ExceptionDlg;

/**
 *
 * @author favdb
 */
public class XmlFile {

	private final String fileName;
	private boolean fileOpened;
	private DocumentBuilder documentBuilder;
	private Document document;
	private Element rootNode;

	public XmlFile(String fileName) {
		this.fileName = fileName;
	}

	public boolean openXml() {
		//LOG.trace("Importer.openXml()");
		fileOpened = false;
		documentBuilder = null;
		try {
			documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch (ParserConfigurationException ex) {
			ExceptionDlg.show("", new Exception(I18N.getMsg("import.dlg.open.docbuilder")));
			LOG.err("DocumentBuilder error:\n", ex);
			return (false);
		}
		document = readDom();
		if (document == null) {
			rootNode = null;
			return (false);
		}
		rootNode = document.getDocumentElement();
		fileOpened = true;
		return (true);
	}

	public Document readDom() {
		//LOG.trace("Importer.readDom()");
		Document rc = null;
		try {
			rc = documentBuilder.parse(new File(fileName));
		} catch (SAXException ex) {
			ExceptionDlg.show("", new Exception(I18N.getMsg("import.dlg.open.parsing")));
			LOG.err("Parsing error for " + fileName, ex);
		} catch (IOException ex) {
			ExceptionDlg.show("", new Exception(I18N.getMsg("import.dlg.open.io")));
			LOG.err("I/O error for " + fileName, ex);
		}
		return (rc);
	}

	public void closeXml() {
		//LOG.trace(TT+".closeXml()");
		if (fileOpened) {
			fileOpened = false;
			document = null;
			documentBuilder = null;
		}
	}

	public static List<String> getAllProperty(String table) {
		//LOG.trace(TT+".getAllProperty(table=" + table + ")");
		List<String> ls = new ArrayList<>();
		String xmlFile = /*"/storybook/model/hbn/entity/"+*/ table + ".hbm.xml";
		String str = IOUtil.resourceRead(xmlFile, Attribute.class);
		try {
			InputSource is = new InputSource(new StringReader(str));
			DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			Document doc = builder.parse(is);
			doc.normalizeDocument();
			Element root = doc.getDocumentElement();
			NodeList children = root.getChildNodes();
			for (int i = 0; i < children.getLength(); i++) {
				if (children.item(i).getNodeType() == Node.ELEMENT_NODE) {
					Element element = (Element) children.item(i);
					String tag = element.getTagName();
					if (tag.equals("class")) {
						String tb = getString(element, "table");
						NodeList props = element.getElementsByTagName("property");
						for (int j = 0; j < props.getLength(); j++) {
							Element propElement = (Element) props.item(j);
							if (!"property".equalsIgnoreCase(propElement.getTagName())) {
								continue;
							}
							String name = getString(propElement, "name");
							String type = getString(propElement, "type");
							String length = getString(propElement, "length");
							if (length.isEmpty()) {
								if (type.contains("String")) {
									length = "255";
								} else {
									length = "0";
								}
							}
							ls.add(tb + "," + name + "," + type + "," + length);
						}
					} else if (!"property".equals(element.getTagName())) {
						continue;
					}

				}
			}
		} catch (ParserConfigurationException | SAXException | IOException ex) {
			LOG.err("Exception error:", ex);
		}
		return (ls);
	}

	public static Integer getInteger(Node node, String key) {
		String x = getString(node, key).trim();
		if (x.isEmpty() || "null".equals(x)) {
			return (0);
		}
		return (Integer.parseInt(x));
	}

	public static String getString(Node node, String key) {
		Element e = (Element) node;
		return (e.getAttribute(key).trim());
	}

}
